from abc import ABC, abstractmethod
from typing import Any, Dict

class HttpRequest:
    def __init__(self, body: Dict[str, Any]) -> None:
        self.body = body
    
    def get(self, param: str) -> 'Any | None':
        return self.body.get(param)


class HttpResponse:
    def __init__(self, status_code: int, body: Dict[str, Any]) -> None:
        self.status_code = status_code
        self.body = body

    @property
    def is_ok(self) -> bool:
        return self.status_code in [200, 201]
    
class Controller(ABC):
    @abstractmethod
    def handle(self, request: HttpRequest) -> HttpResponse:
        pass


from clusternet.apis.presentation.helpers import error, internal_server_error, success
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname


class StopWorkerController(Controller):
    def __init__(self) -> None:
        self.net = WorkerInstance.instance()
    
    def handle(self, request: HttpRequest) -> HttpResponse:
        hostname = get_hostname()

        try:
            if(not self.net.is_running):
                self.net.stop_hosts()
                WorkerInstance.clear_instance()
                raise Exception(f'[{hostname}]: Containernet already is stopped')

            self.net.stop()
            WorkerInstance.clear_instance()
        except Exception as ex:
            message = f'{ex}'
            return internal_server_error(error(message))
        
        return success({'content': f'[{hostname}]: Containernet stopped'})
            


from typing import Any, List
import docker
import socket

from mininet.log import info
from clusternet.apis.worker.data import WorkerInstance


def get_hostname() -> str:
    return socket.gethostname()


def clean_containers_with_prefix(prefix: str) -> List[str]:
    client = docker.from_env()
    removed_containers = []

    for container in client.containers.list(all=True):
        name = str(container.name) # type: ignore
        if(name.startswith(prefix)):
            info(f'*** Cleaning container {name}\n')
            container.remove(force=True) # type: ignore
            removed_containers.append(name)
    
    return removed_containers

def run_container(name: str, image: str, **params: Any):
    client = docker.from_env()
    client.containers.run(image, name=name, remove=True, detach=True, **params)



from flask.wrappers import Request

from typing import Any, Dict, List

from clusternet.apis.presentation.exceptions import BadRequestException
from clusternet.apis.presentation.protocols import HttpRequest, HttpResponse


def error(message: str = 'Internal server error') -> Dict[str, str]:
    return {'error': message}

def bad_request(body: Dict[str, Any]) -> HttpResponse:
    return HttpResponse(status_code=400, body=body)

def created(body: Dict[str, Any]) -> HttpResponse:
    return HttpResponse(status_code=201, body=body)

def internal_server_error(body: Dict[str, Any]) -> HttpResponse:
    return HttpResponse(status_code=500, body=body)

def not_found(body: Dict[str, Any]) -> HttpResponse:
    return HttpResponse(status_code=404, body=body)

def success(body: Dict[str, Any]) -> HttpResponse:
    return HttpResponse(status_code=200, body=body)


def validate_required_params(request: HttpRequest, params: List[str]):
    for param in params:
        if(request.get(param) is None):
            raise BadRequestException(f'Missing param {param}')


def parse_request(request: Request) -> HttpRequest:
    try: 
        body = request.json
        if(body is None): raise
        
        return HttpRequest(body=body) 
    
    except:
        return HttpRequest(body={})
    

    


import os
from typing import List

from clusternet.client.worker import RemoteWorker

from clusternet.monitoring.helpers import (
    create_grafana_files, create_prometheus_file, resolve_ip
)
from clusternet.monitoring.services import (
    CLUSTERNET_PREFIX, 
    clean_containers, clean_workers, run_cadvisor, 
    run_grafana, run_node_exporters, run_prometheus
)

DATA_PATH = '/tmp/fogbed'
GRAFANA_PATH = f'{DATA_PATH}/grafana'
PROMETHEUS_PATH = f'{DATA_PATH}/prometheus'


class ClusterMonitoring:
    def __init__(self, 
        monitor_server: str, 
        scrape_interval: int = 1,
        grafana_uid: str = 'clusternet',
        workers: 'List[RemoteWorker] | None' = None
    ):
        self.monitor_server = monitor_server
        self.scrape_interval = scrape_interval
        self.grafana_uid = grafana_uid
        self.workers = workers


    def get_ip_targets(self) -> List[str]:
        if(self.workers is None):
            return [self.monitor_server]
        return [worker.ip for worker in self.workers]


    def create_data_folders(self, directories: List[str]):
        for directory in directories:
            os.makedirs(directory, exist_ok=True)
        
        monitor_server = resolve_ip(self.monitor_server)
        create_grafana_files(
            grafana_path=GRAFANA_PATH,
            grafana_uid=self.grafana_uid,
            prometheus_url=f'http://{monitor_server}:9090')
        create_prometheus_file(
            prometheus_path=PROMETHEUS_PATH,
            scrape_interval=self.scrape_interval,
            workers=self.get_ip_targets())

    
    def stop(self):
        clean_containers(prefix=CLUSTERNET_PREFIX)
        if(self.workers is not None):
            clean_workers(self.workers)

    
    def start(self):
        self.create_data_folders([GRAFANA_PATH, PROMETHEUS_PATH])
        
        if(self.workers is None):
            run_cadvisor()
        else:
            run_node_exporters(self.workers)

        run_prometheus(PROMETHEUS_PATH)
        run_grafana(GRAFANA_PATH)        
        print(f'[{self.monitor_server}]: Prometheus address http://localhost:9090/graph')
        print(f'[{self.monitor_server}]: Grafana address http://localhost:3000/d/{self.grafana_uid}/docker-monitoring')



from clusternet.apis.presentation.helpers import error, internal_server_error, success
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname

class RunPingallController(Controller):
    def __init__(self) -> None:
        self.net = WorkerInstance.instance()
    
    def handle(self, request: HttpRequest) -> HttpResponse:
        hostname = get_hostname()

        try:
            if(not self.net.is_running):
                raise Exception(f'[{hostname}]: Containernet is not running')

            packets = self.net.pingAll()
            
            return success({'content': f'{packets}% dropped', 'dropped': packets})
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))
    


import httpx


class RemoteContainer:
    def __init__(self, name: str, url: str) -> None:
        self.name = name
        self.url = url

    def cmd(self, command: str) -> str:
        data = {'command': command}
        response = httpx.post(url=f'{self.url}/hosts/{self.name}/cmd', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error']) 
        return response.json()['content']
    
    
    def config_default(self):
        response = httpx.get(url=f'{self.url}/hosts/{self.name}/config', timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
    

    def get_ip(self) -> str:
        response = httpx.get(url=f'{self.url}/containers/{self.name}/ip', timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        return response.json()['content']


    def update_cpu(self, cpu_quota: int, cpu_period: int):
        data = {'cpu_quota': cpu_quota, 'cpu_period': cpu_period}
        response = httpx.put(url=f'{self.url}/containers/{self.name}/cpu', json=data, timeout=None)

        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
    

    def update_memory(self, mem_limit: int):
        data = {'mem_limit': mem_limit}
        response = httpx.put(url=f'{self.url}/containers/{self.name}/memory', json=data, timeout=None)

        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')


    def start(self):
        response = httpx.get(url=f'{self.url}/containers/{self.name}/start', timeout=None)

        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
    

    def stop(self):
        response = httpx.get(url=f'{self.url}/containers/{self.name}/stop', timeout=None)

        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
        


from clusternet.apis.presentation.exceptions import BadRequestException
from clusternet.apis.presentation.helpers import (
    bad_request, error, internal_server_error, success, validate_required_params
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import (
    get_hostname, clean_containers_with_prefix, WorkerInstance
)


class CleanContainersController(Controller):
    def __init__(self) -> None:
        pass
    
    def handle(self, request: HttpRequest) -> HttpResponse:
        try:
            validate_required_params(request, ['containers_prefix'])
            prefix = str(request.body['containers_prefix'])
            containers = clean_containers_with_prefix(prefix)
            
            WorkerInstance.clear_instance()
            return success({'content': f'[{get_hostname()}]: Cleaned containers: {containers}'})
        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))


from clusternet.apis.presentation.exceptions import BadRequestException
from clusternet.apis.presentation.helpers import (
    bad_request, error, internal_server_error, success, validate_required_params
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname

class RemoveLinkController(Controller):
    def __init__(self) -> None:
        self.net = WorkerInstance.instance()

    def handle(self, request: HttpRequest) -> HttpResponse:
        required_params = ['node1', 'node2']
        hostname = get_hostname()

        try:
            validate_required_params(request, required_params)
            node1 = str(request.body['node1'])
            node2 = str(request.body['node2'])
            self.net.removeLink(node1=node1, node2=node2)

            return success({'content': f'[{hostname}]: link removed between {node1} and {node2}'})

        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))



import argparse
import os

from flask import Flask, jsonify, request
from flask.wrappers import Request

from clusternet.apis.worker.controllers import (
    AddController, AddDockerController, AddLinkController, AddSwitchController,
    CleanContainersController, ConfigDefaultController, GetDockerIPController,
    RemoveDockerController, RemoveLinkController, RunCommandOnHostController, 
    RunServiceController, RunPingallController, StartDockerController, 
    StopDockerController, StartWorkerController, StopWorkerController, 
    UpdateCPUController, UpdateMemoryController
)
from clusternet.apis.presentation.helpers import parse_request
from clusternet.apis.presentation.protocols import Controller


server = Flask(__name__)


def make_response(controller: Controller, request: Request):
    response = controller.handle(parse_request(request))
    return jsonify(response.body), response.status_code

@server.route('/controllers', methods=['POST'])
def add_controller():
    controller = AddController()
    return make_response(controller, request)


@server.route('/containers', methods=['POST'])
def add_docker():
    controller = AddDockerController()
    return make_response(controller, request)


@server.route('/containers/clean', methods=['POST'])
def clean_containers():
    controller = CleanContainersController()
    return make_response(controller, request)


@server.route('/containers/<string:name>/ip', methods=['GET'])
def get_ip(name: str):
    controller = GetDockerIPController(name)
    return make_response(controller, request)


@server.route('/containers/<string:name>/start', methods=['GET'])
def start_docker(name: str):
    controller = StartDockerController(name)
    return make_response(controller, request)


@server.route('/containers/<string:name>/stop', methods=['GET'])
def stop_docker(name: str):
    controller = StopDockerController(name)
    return make_response(controller, request)


@server.route('/containers/<string:name>', methods=['DELETE'])
def remove_docker(name: str):
    controller = RemoveDockerController(name)
    return make_response(controller, request)


@server.route('/containers/<string:name>/cpu', methods=['PUT'])
def update_cpu(name: str):
    controller = UpdateCPUController(name)
    return make_response(controller, request)


@server.route('/containers/<string:name>/memory', methods=['PUT'])
def update_memory(name: str):
    controller = UpdateMemoryController(name)
    return make_response(controller, request)


@server.route('/links', methods=['POST'])
def add_link():
    controller = AddLinkController()
    return make_response(controller, request)


@server.route('/links/remove', methods=['POST'])
def remove_link():
    controller = RemoveLinkController()
    return make_response(controller, request)


@server.route('/switches', methods=['POST'])
def add_switch():
    controller = AddSwitchController()
    return make_response(controller, request)


@server.route('/hosts/<string:name>/config', methods=['GET'])
def config_default(name: str):
    controller = ConfigDefaultController(name)
    return make_response(controller, request)


@server.route('/hosts/<string:name>/cmd', methods=['POST'])
def run_command(name: str):
    controller = RunCommandOnHostController(name)
    return make_response(controller, request)


@server.route('/hosts/pingall', methods=['GET'])
def run_pingall():
    controller = RunPingallController()
    return make_response(controller, request)


@server.route('/services', methods=['POST'])
def run_service():
    controller = RunServiceController()
    return make_response(controller, request)


@server.route('/start', methods=['GET'])
def start():
    controller = StartWorkerController()
    return make_response(controller, request)


@server.route('/stop', methods=['GET'])
def stop():
    controller = StopWorkerController()
    return make_response(controller, request)


def main():
    parser = argparse.ArgumentParser('sudo RunWorker')
    parser.add_argument('-p', '--port', type=int, default=5000, help='run server on especified port (default: 5000)')
    args = parser.parse_args()
    
    server.run(host='0.0.0.0', port=int(os.environ.get('PORT', args.port)))

if(__name__=='__main__'):
    main()
    


from clusternet.apis.presentation.exceptions import NotFoundException
from clusternet.apis.presentation.helpers import (
    error, internal_server_error, not_found, success
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname

class RemoveDockerController(Controller):
    def __init__(self, name: str) -> None:
        self.name = name
        self.net  = WorkerInstance.instance()

    def handle(self, request: HttpRequest) -> HttpResponse:
        name = self.name
        hostname = get_hostname()

        try:
            if(not self.net.removeDocker(name)):
                raise NotFoundException(f'[{hostname}]: container {name} not found')

            return success({'content': f'[{hostname}]: container {name} removed'})

        except NotFoundException as ex:
            return not_found(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))


from mininet.net import Containernet
from mininet.node import Docker, Host
from mininet.log import info

class ContainernetWorker(Containernet):
    def __init__(self, **params):
        super().__init__(**params)
        self.is_running = False
    
    def getDocker(self, name: str) -> Docker:
        return self[name]

    def getHost(self, name: str) -> Host:
        return self[name]

    def start(self):
        super().start()
        self.is_running = True

    def stop(self):
        super().stop()
        self.is_running = False

    def stop_hosts(self):
        info(f'*** Stopping {len(self.hosts)} hosts\n')
        for host in self.hosts:
            info(host.name + ' ')
            host.terminate()
        info('\n')


class WorkerInstance:
    worker = ContainernetWorker(topo=None, build=False)

    @staticmethod
    def instance():
        return WorkerInstance.worker

    @staticmethod
    def clear_instance():
        WorkerInstance.worker = ContainernetWorker(topo=None, build=False)


from clusternet.apis.presentation.exceptions import BadRequestException, NotFoundException
from clusternet.apis.presentation.helpers import bad_request, error, internal_server_error, not_found, success, validate_required_params
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname


class UpdateMemoryController(Controller):
    def __init__(self, name: str) -> None:
        self.name = name
        self.net  = WorkerInstance.instance()

    def handle(self, request: HttpRequest) -> HttpResponse:
        hostname = get_hostname()
        required_params = ['mem_limit']

        try:
            if(not self.name in self.net):
                raise NotFoundException(f'[{hostname}]: container {self.name} not found')

            validate_required_params(request, required_params)
            mem_limit = int(request.body['mem_limit'])
            
            docker = self.net.getDocker(name=self.name)
            docker.updateMemoryLimit(mem_limit)

            return success({'content': f'[{hostname}]: container {self.name} memory updated'})
            
        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except NotFoundException as ex:
            return not_found(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))



from functools import partial

from clusternet.apis.presentation.exceptions import BadRequestException
from clusternet.apis.presentation.helpers import (
    bad_request, created, error, internal_server_error, validate_required_params
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname

from mininet.node import RemoteController

class AddController(Controller):
    def __init__(self) -> None:
        self.net = WorkerInstance.instance()

    def handle(self, request: HttpRequest) -> HttpResponse:
        required_params = ['name', 'ip', 'port']
        hostname = get_hostname()

        try:
            validate_required_params(request, required_params)
            name = str(request.body['name'])
            ip   = str(request.body['ip'])
            port = int(request.body['port'])

            if(name in self.net):
                raise Exception(f'[{hostname}]: controller {name} already exist')

            controller = partial(RemoteController, ip=ip, port=port)
            self.net.addController(name, controller)
            return created({'content': f'[{hostname}]: controller {name} created'})

        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))



from clusternet.apis.worker.controllers.add_controller import AddController
from clusternet.apis.worker.controllers.add_docker import AddDockerController
from clusternet.apis.worker.controllers.add_link import AddLinkController
from clusternet.apis.worker.controllers.add_switch import AddSwitchController
from clusternet.apis.worker.controllers.clean_containers import CleanContainersController
from clusternet.apis.worker.controllers.config_default import ConfigDefaultController
from clusternet.apis.worker.controllers.get_docker_ip import GetDockerIPController
from clusternet.apis.worker.controllers.remove_docker import RemoveDockerController
from clusternet.apis.worker.controllers.remove_link import RemoveLinkController
from clusternet.apis.worker.controllers.run_command import RunCommandOnHostController
from clusternet.apis.worker.controllers.run_pingall import RunPingallController
from clusternet.apis.worker.controllers.run_service import RunServiceController
from clusternet.apis.worker.controllers.start_docker import StartDockerController
from clusternet.apis.worker.controllers.start_worker import StartWorkerController
from clusternet.apis.worker.controllers.stop_docker import StopDockerController
from clusternet.apis.worker.controllers.stop_worker import StopWorkerController
from clusternet.apis.worker.controllers.update_cpu import UpdateCPUController
from clusternet.apis.worker.controllers.update_memory import UpdateMemoryController




from clusternet.apis.presentation.exceptions import BadRequestException
from clusternet.apis.presentation.helpers import (
    bad_request, created, error, internal_server_error, validate_required_params
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import get_hostname, run_container


class RunServiceController(Controller):
    def __init__(self) -> None:
        pass
    
    def handle(self, request: HttpRequest) -> HttpResponse:
        required_params = ['name', 'image']
        try:
            hostname = get_hostname()
            validate_required_params(request, required_params)
            
            name = str(request.body.pop('name'))
            image = str(request.body.pop('image'))
            run_container(name, image, **request.body)
            
            return created({'content': f'[{hostname}]: service {name} created'})
        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))



from clusternet.apis.presentation.exceptions import NotFoundException
from clusternet.apis.presentation.helpers import (
    error, internal_server_error, not_found, success
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname


class StopDockerController(Controller):
    def __init__(self, name: str) -> None:
        self.name = name
        self.net = WorkerInstance.instance()

    def handle(self, request: HttpRequest) -> HttpResponse:   
        hostname = get_hostname()
        
        try:
            if(not self.name in self.net):
                raise NotFoundException(f'[{hostname}]: container {self.name} not found')
            
            self.net.getDocker(self.name).stop()
            return success({'content': f'[{hostname}]: container {self.name} stopped'})

        except NotFoundException as ex:
            return not_found(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))


from clusternet.apis.presentation.exceptions import BadRequestException
from clusternet.apis.presentation.helpers import (
    bad_request, created, error, internal_server_error, validate_required_params
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname

class AddDockerController(Controller):
    def __init__(self) -> None:
        self.net = WorkerInstance.instance()
    
    def handle(self, request: HttpRequest) -> HttpResponse:
        required_params = ['name']
        hostname = get_hostname()

        try: 
            validate_required_params(request, required_params)
            name = str(request.body['name'])

            if(name in self.net):
                raise Exception(f'[{hostname}]: container {name} already exist')
             
            self.net.addDocker(**request.body)
            return created({'content': f'[{hostname}]: container {name} created'})
        
        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))
        


from typing import Any
import httpx

from clusternet.client.container import RemoteContainer

class RemoteWorker:
    def __init__(self, ip: str, port: int = 5000) -> None:
        self.ip         = ip
        self.url        = f'http://{ip}:{port}'
        self.is_running = False


    def add_controller(self, name: str, ip: str, port: int):
        data = {'name': name, 'ip': ip, 'port': port}
        response = httpx.post(url=f'{self.url}/controllers', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')

    
    def add_docker(self, name: str, **params: Any) -> RemoteContainer:
        data = {'name': name, **params}
        response = httpx.post(url=f'{self.url}/containers', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])

        print(f'{response.json()["content"]}')
        return RemoteContainer(name, self.url)


    def add_link(self, node1: str, node2: str, **params: Any):
        data = {'node1': node1, 'node2': node2, **params}
        response = httpx.post(url=f'{self.url}/links', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')


    def add_switch(self, name: str):
        data = {'name': name}
        response = httpx.post(url=f'{self.url}/switches', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
    

    def clean_containers(self, prefix: str):
        data = {'containers_prefix': prefix}
        response = httpx.post(url=f'{self.url}/containers/clean', json=data, timeout=None)

        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
        

    def config_default(self, name: str):
        response = httpx.get(url=f'{self.url}/hosts/{name}/config', timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
    

    def remove_docker(self, name: str):
        response = httpx.delete(url=f'{self.url}/containers/{name}', timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')


    def remove_link(self, node1: str, node2: str):
        data = {'node1': node1, 'node2': node2}
        response = httpx.post(url=f'{self.url}/links/remove', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')


    def run_command(self, name: str, command: str):
        data = {'command': command}
        response = httpx.post(url=f'{self.url}/hosts/{name}/cmd', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
    

    def run_pingall(self):
        response = httpx.get(url=f'{self.url}/hosts/pingall', timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')


    def run_service(self, name: str, image: str, **params: Any):
        data = {'name': name, 'image': image, **params}
        response = httpx.post(url=f'{self.url}/services', json=data, timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        print(f'{response.json()["content"]}')
    

    def start(self):
        response = httpx.get(url=f'{self.url}/start', timeout=None)
        
        if(response.is_error):
            raise Exception(response.json()['error'])
        self.is_running = True
        print(f'{response.json()["content"]}')


    def stop(self):
        response = httpx.get(url=f'{self.url}/stop', timeout=None)
        
        if(response.is_error):
            print(response.json()['error'])
        else:
            self.is_running = False
            print(f'{response.json()["content"]}')



import docker
from typing import Any, List

from clusternet.client.worker import RemoteWorker

try:
    client = docker.from_env()
except:
    pass

CLUSTERNET_PREFIX = 'cn.'

def clean_containers(prefix: str):
    for container in client.containers.list(all=True):
        name = str(container.name) # type: ignore
        if(name.startswith(prefix)):
            container.remove(force=True) # type: ignore


def run_container(name: str, image: str, **params: Any):
    client.containers.run(image, name=name, remove=True, detach=True, **params)


def run_cadvisor():
    run_container(
        name=CLUSTERNET_PREFIX + 'cadvisor',
        image='gcr.io/cadvisor/cadvisor:v0.47.2',
        ports={'8080/tcp': 8080},
        volumes=[
            '/:/rootfs:ro',
            '/var/run:/var/run:ro',
            '/sys:/sys:ro',
            '/var/lib/docker/:/var/lib/docker:ro',
            '/dev/disk/:/dev/disk:ro'],
        devices=['/dev/kmsg'],
        privileged=True)


def run_grafana(grafana_path: str):
    run_container(
        name=CLUSTERNET_PREFIX + 'grafana',
        image='grafana/grafana-enterprise',
        ports={'3000/tcp': 3000},
        volumes=[f'{grafana_path}:/etc/grafana'],
        environment={
            'GF_PATHS_PROVISIONING': '/etc/grafana',
            'GF_SECURITY_ADMIN_PASSWORD': 'admin123'})


def run_prometheus(prometheus_path: str):
    run_container(
        name=CLUSTERNET_PREFIX + 'prometheus',
        image='prom/prometheus',
        ports={'9090/tcp': 9090},
        volumes=[f'{prometheus_path}:/etc/prometheus'])


def run_node_exporters(workers: List[RemoteWorker]):
    for worker in workers:
        worker.run_service(
            name=CLUSTERNET_PREFIX + 'cadvisor',
            image='gcr.io/cadvisor/cadvisor:v0.47.2',
            ports={'8080/tcp': 8080},
            volumes=[
                '/:/rootfs:ro',
                '/var/run:/var/run:ro',
                '/sys:/sys:ro',
                '/var/lib/docker/:/var/lib/docker:ro',
                '/dev/disk/:/dev/disk:ro'],
            devices=['/dev/kmsg'],
            privileged=True)


def clean_workers(workers: List[RemoteWorker]):
    for worker in workers:
        worker.clean_containers(prefix=CLUSTERNET_PREFIX)



from clusternet.apis.presentation.exceptions import BadRequestException, NotFoundException
from clusternet.apis.presentation.helpers import (
    bad_request, error, internal_server_error, not_found, success, validate_required_params
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname


class RunCommandOnHostController(Controller):
    def __init__(self, name: str) -> None:
        self.net = WorkerInstance.instance()
        self.name = name
    
    def handle(self, request: HttpRequest) -> HttpResponse:
        hostname = get_hostname()
        required_params = ['command']

        try:        
            validate_required_params(request, required_params)

            if(not self.name in self.net):
                raise NotFoundException(f'[{hostname}]: node {self.name} not found')

            command = str(request.body['command'])
            host    = self.net.getHost(self.name)
            output  = host.cmd(command)

            return success({'content': output})
            
        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except NotFoundException as ex:
            return not_found(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))



from setuptools import setup, find_packages

with open('README.md', 'r') as file:
    description = file.read()

setup(
    name='clusternet',
    version='0.9.3',
    description='Distributed Software Defined Network Emulation',
    long_description=description,
    long_description_content_type='text/markdown',
    keywords=['networking', 'emulator', 'containernet', 'mininet', 'OpenFlow', 'SDN', 'fog'],
    url='https://github.com/EsauM10/clusternet',
    author='Esaú Mascarenhas',
    author_email='esaumasc@gmail.com',
    classifiers=[
        'Intended Audience :: Developers'
    ],
    packages=find_packages(),
    install_requires = [
        'Flask',
        'Flask-Cors',
        'httpx',
        'pyyaml'
    ],
    entry_points={
        'console_scripts': [
            'RunWorker = clusternet.server.worker_app:main',
        ]
    },
    include_package_data=True,
    zip_safe=False
)



from pathlib import Path
import shutil
import socket
from typing import List
import yaml


BASEDIR = str(Path(__file__).parent.parent)


def get_file_content(filename: str) -> str:
    with open(filename, mode='r', encoding='utf-8') as file:
        return file.read()

def save_file_content(filename: str, content: str):
    with open(filename, mode='w', encoding='utf-8') as file:
        file.write(content)

def resolve_ip(ip: str) -> str:
    return socket.gethostbyname(ip)

def create_jobs(workers: List[str], scrape_interval: int):
    jobs = []
    for worker in workers:
        jobs.append({
            'job_name': worker, 
            'scrape_interval': f'{scrape_interval}s', 
            'static_configs': [{'targets': [f'{resolve_ip(worker)}:8080']}]
        })
    return jobs


def update_datasource(grafana_path: str, grafana_uid: str, prometheus_url: str):
    content = get_file_content(f'{grafana_path}/datasources/datasource.yml')
    datasource = content.replace('$PROMETHEUS_URL', prometheus_url).replace('$UID', grafana_uid)
    save_file_content(f'{grafana_path}/datasources/datasource.yml', datasource)


def update_dashboard_data(grafana_path: str, grafana_uid: str):
    content = get_file_content(f'{grafana_path}/dashboards/docker.json')
    dashboard_data = content.replace('$UID', grafana_uid)
    save_file_content(f'{grafana_path}/dashboards/docker.json', dashboard_data)


def create_grafana_files(grafana_path: str, grafana_uid: str, prometheus_url: str):
    shutil.copytree(f'{BASEDIR}/grafana', grafana_path, dirs_exist_ok=True)
    update_datasource(grafana_path, grafana_uid, prometheus_url)
    update_dashboard_data(grafana_path, grafana_uid)
    

def create_prometheus_file(prometheus_path: str, workers: List[str], scrape_interval: int):
    jobs = create_jobs(workers, scrape_interval)
    content = yaml.safe_dump({'scrape_configs': jobs})
    save_file_content(f'{prometheus_path}/prometheus.yml', content)



from typing import List
from mininet.link import TCLink
from clusternet.apis.presentation.exceptions import BadRequestException, NotFoundException
from clusternet.apis.presentation.helpers import (
    bad_request, created, error, internal_server_error, not_found, validate_required_params
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname

class AddLinkController(Controller):
    def __init__(self) -> None:
        self.net = WorkerInstance.instance()

    def verify_if_node_exist(self, nodes: List[str]):
        for node in nodes:
            if(not node in self.net): 
                raise NotFoundException(f'[{get_hostname()}]: node {node} not found')

    def handle(self, request: HttpRequest) -> HttpResponse:
        required_params = ['node1', 'node2']
        hostname = get_hostname()

        try:
            validate_required_params(request, required_params)
            node1 = str(request.body['node1'])
            node2 = str(request.body['node2'])

            self.verify_if_node_exist(nodes=[node1, node2])
            
            if(self.net.linksBetween(node1, node2)):
                raise Exception(f'[{hostname}]: already exist a link between {node1} and {node2}')

            self.net.addLink(**request.body, cls=TCLink)
            
            return created({'content': f'[{hostname}]: link created between {node1} and {node2}'})

        except BadRequestException as ex:
            return bad_request(error(f'{ex}'))
        except NotFoundException as ex:
            return not_found(error(f'{ex}'))
        except Exception as ex:
            return internal_server_error(error(f'{ex}'))


from clusternet.apis.presentation.helpers import (
    error, internal_server_error, success
)
from clusternet.apis.presentation.protocols import Controller, HttpRequest, HttpResponse
from clusternet.apis.worker.helpers import WorkerInstance, get_hostname


class StartWorkerController(Controller):
    def __init__(self) -> None:
        self.net = WorkerInstance.instance()

    def handle(self, request: HttpRequest) -> HttpResponse:   
        hostname = get_hostname()
        
        try:
            if(self.net.is_running):
                raise Exception(f'[{hostname}]: Containernet already is running')
            
            self.net.start()
            return success({'content': f'[{hostname}]: Containernet started'})

        except Exception as ex:
            return internal_server_error(error(f'{ex}'))
        